#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <SpiceUsr.h>

#define STRLEN 100

extern void rdplat_ ( SpiceChar [], SpiceInt );
extern void initfv_ ( SpiceInt*, SpiceInt*, SpiceChar*, SpiceDouble[], SpiceInt*, SpiceDouble[4][3], SpiceDouble[4][3] );
extern void pltot_ ( SpiceInt*, SpiceInt*, SpiceInt* );
extern void pntocc_ ( SpiceInt*, SpiceInt*, SpiceDouble[4][3], SpiceDouble[4][3], SpiceDouble[], SpiceDouble[], SpiceDouble*, SpiceDouble*, SpiceDouble*, SpiceBoolean* );
extern void plinfo_ ( SpiceInt*, SpiceInt*, SpiceInt[], SpiceDouble[3][3], SpiceDouble[], SpiceDouble[], SpiceDouble* );
extern void plbore_ ( SpiceDouble [], SpiceDouble [], SpiceInt*, SpiceInt*, SpiceDouble [], SpiceBoolean* );

void furnsh_kernels ( void );

int main ( int argc, char **argv ) {

  /* Definition of variables */

  SpiceInt     instid = -130200;
  SpiceInt     body   = 2025143;
  
  SpiceDouble  et, lt;
  SpiceInt     fvroom = 4, fovsiz;
  SpiceDouble  bsight[3], fov[4][3], fovnrm[4][3];
  SpiceChar    frame[STRLEN];
  SpiceDouble  obs[3], sun[3], toobs[3], tosun[3];
  SpiceDouble  rotate[3][3];
  
  SpiceInt     maxbds, nplat, nvert, pldex;
  SpiceDouble  fovamt, visamt, litamt;
  SpiceBoolean away;

  SpiceInt     vids[3];
  SpiceDouble  albedo, normal[3], center[3], verts[3][3];
  SpiceDouble  v1[3], v2[3], vup[3], norm[3], vupmag;
  SpiceDouble  inc, emi, pha, area;

  int          i;

  
  /* Beginning of the main function */
  
  if ( argc != 2 ) {
    fprintf ( stderr, "Usage:%s ephemeris_time [s]\n", argv[0] );
    exit ( 0 ); }
  et = atof ( argv[1] );
  
  /* Furnish the necessary spice kernels. */
  furnsh_kernels ();

  /* Determine the number of bodies that have plates models loaded. */
  pltot_ ( &maxbds, &nplat, &nvert );
  if ( maxbds != 1 ) {
    fprintf ( stderr, "ERROR: failed to furnish plate model kernel\n" );
	exit ( 0 ); }  

  /*
   * We need the state of the observer in the apparent bodyfixed
   * frame of the body. We also need the state of the sun
   * in the apparent body fixed frame relative to the body
   * corrected for light time to the body from the observer.
  */
  spkpos_c ( "SUN", et-lt, "ITOKAWA_FIXED", "LT+S", "2025143", sun, &lt );
  spkpos_c ( "2025143", et, "ITOKAWA_FIXED", "LT+S", "-130", obs, &lt );
  vminus_c ( obs, obs );

  /* Obtain field of view parameters and side normals. */
  initfv_ ( &instid, &fvroom, frame, bsight, &fovsiz, fov, fovnrm );

  /* Get the transformation that transforms vectors in FRAME
	 to the bodyfixed frame of this body. */
  pxform_c ( frame, "ITOKAWA_FIXED", et, rotate );
  mxv_c ( rotate, bsight, bsight );  
  for ( i = 0 ; i < fovsiz ; i++ ) {
	mxv_c ( rotate, fov[i], fov[i] );
	mxv_c ( rotate, fovnrm[i], fovnrm[i] );
  }

  printf ( "# plate-id, projected_area [m^2], " );
  printf ( "incidence [deg], emission [deg], phase [deg]\n" );

  /* Now examine each plate for this body. */
  for ( pldex = 1 ; pldex <= nplat ; pldex++ ) {

	pntocc_ ( &pldex, &fovsiz, fov, fovnrm, obs, sun,
			  &fovamt, &visamt, &litamt, &away );

	if ( visamt > 0.5 && litamt > 0.5 ) {

	  plinfo_ ( &pldex, &body, vids, verts, normal, center, &albedo );

	  /* Get the vector from the plate's center to the observer */
	  vsub_c ( obs, center, toobs );
	  /* Get the vector from the plate's center to the sun */
	  vsub_c ( sun, center, tosun );

	  inc = vsep_c ( tosun, normal );
	  emi = vsep_c ( toobs, normal );
	  pha = vsep_c ( tosun, toobs );

	  vsub_c ( verts[1], verts[0], v1 );
	  vsub_c ( verts[2], verts[0], v2 );
	  vcrss_c ( v1, v2, vup );
	  unorm_c ( vup, norm, &vupmag );
	  area = 0.5 * vupmag * cos ( emi ) * 1.0E+06;
	  
	  inc *= dpr_c (); emi *= dpr_c (); pha *= dpr_c ();
	  
	  printf ( "%ld\t%.6e\t%.6e\t%.6e\t%.6e\n",
			   pldex, area, inc, emi, pha );
	}
  }

  return ( 0 );
}

void furnsh_kernels ( void ) {
  
  SpiceChar *LSK, *PCK, *SPK;
  SpiceChar *ITOKAWA_FK;
  SpiceChar *ITOKAWA_PCK;
  SpiceChar *ITOKAWA_PMK;
  SpiceChar *ITOKAWA_SPK;
  SpiceChar *HAYABUSA_CK;
  SpiceChar *HAYABUSA_FK;
  SpiceChar *HAYABUSA_IK;
  SpiceChar *HAYABUSA_SCLK;
  SpiceChar *HAYABUSA_SPK;
  
  LSK  = getenv ( "PATH_LSK" );
  PCK  = getenv ( "PATH_PCK" );
  SPK  = getenv ( "PATH_SPK" );
  ITOKAWA_FK   = getenv ( "PATH_ITOKAWA_FK" );
  ITOKAWA_PCK  = getenv ( "PATH_ITOKAWA_PCK" );
  ITOKAWA_PMK  = getenv ( "PATH_ITOKAWA_PMK" );
  ITOKAWA_SPK  = getenv ( "PATH_ITOKAWA_SPK" );
  HAYABUSA_CK  = getenv ( "PATH_HAYABUSA_CK" );
  HAYABUSA_FK  = getenv ( "PATH_HAYABUSA_FK" );
  HAYABUSA_IK  = getenv ( "PATH_HAYABUSA_IK" );
  HAYABUSA_SCLK= getenv ( "PATH_HAYABUSA_SCLK" );
  HAYABUSA_SPK = getenv ( "PATH_HAYABUSA_SPK" );
  
  furnsh_c ( LSK );
  furnsh_c ( PCK );
  furnsh_c ( SPK );
  furnsh_c ( ITOKAWA_FK );
  furnsh_c ( ITOKAWA_PCK );
  rdplat_  ( ITOKAWA_PMK, STRLEN );
  furnsh_c ( ITOKAWA_SPK );
  furnsh_c ( HAYABUSA_CK );
  furnsh_c ( HAYABUSA_FK );
  furnsh_c ( HAYABUSA_IK );
  furnsh_c ( HAYABUSA_SCLK );
  furnsh_c ( HAYABUSA_SPK );

}
